package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.repository.shipping.MesWorkOrderToWmsRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-05-09 10:54
 * @Modify:
 **/

@Service
@Slf4j
public class MesWorkOrderService implements IMesWorkOrderService {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesWorkOrderService.class);
    @Autowired
    private MesWorkOrderRepository workOrderRepository;
    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesPartRepository mesPartRDao;
    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;
    @Autowired
    private MesProductOffLineRepository mesProductOffLineRDao;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRao;
    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesWorkOrderLogRepository mesWorkOrderLogRepository;
    @Autowired
    private MesWorkOrderToWmsRepository mesWorkOrderToWmsRDao;
    @Autowired
    private MesShiftRepository mesShiftRepository;
    @Autowired
    private IMesConfigService configService;
    @Override
    public MesWorkOrder queryMesWorkOrder(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);
        MesWorkOrder result = workOrderRepository.getByProperty(packBean);

        if (StringUtils.isEmpty(result)) {
            throw new ImppBusiException("查询工单不存在，请检查数据");
        }

        return result;
    }

    @Override
    public List<MesWorkOrder> queryMesWorkOrderList(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);

        return workOrderRepository.findByHqlWhere(packBean);
    }

    @Override
    public ListPager<MesWorkOrder> queryMesWorkOrderListByPager(MesWorkOrder workOrder, Pager pager) {


        DdlPackBean packBean = getDdlPackBean(workOrder);

        pager = PagerHelper.getPager(pager, workOrderRepository.findByHqlWhereCount(packBean));

        List<MesWorkOrder> resultList = workOrderRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    private DdlPackBean getDdlPackBean(MesWorkOrder workOrder) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workOrder.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkOrderNo(), "workOrderNo", packBean);

        if (!StringUtils.isEmpty(workOrder.getWorkOrderStatus())) DdlPreparedPack.getNumEqualPack(workOrder.getStatusList(), "workOrderStatus", packBean);
        else DdlPreparedPack.getInPackList(Stream.of(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(), MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()).collect(Collectors.toList()), "workOrderStatus", packBean);

        DdlPreparedPack.getStringBiggerPack(workOrder.getStartTimeStart(), "planStartTime", packBean);
        DdlPreparedPack.getStringSmallerPack(workOrder.getStartTimeEnd(), "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerPack(workOrder.getEndTimeStart(), "planEndTime", packBean);
        DdlPreparedPack.getStringSmallerPack(workOrder.getEndTimeEnd(), "planEndTime", packBean);

        return packBean;
    }


    @Override
    public void doProductReport(List<MesProduceSn> mesProduceSnList, String organizeCode, String userName) {
        //新增初始化 todo
        Map<String, List<MesProduceSn>> mesWorkOrderMap = mesProduceSnList.stream().collect(Collectors.groupingBy(MesProduceSn::getPartNo));
        Map<String,MesPart> mesPartMap = new HashMap<>();
        MesPart mesPart;
        //查询工单状态
        Integer[] orderStatus =new Integer[]{MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(),MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()};
        for (Map.Entry<String, List<MesProduceSn>> mesProduceSn : mesWorkOrderMap.entrySet()) {
            List<MesProduceSn> mesWorkOrderList = mesProduceSn.getValue();
            //获取物料信息
            if(mesPartMap.containsKey(mesProduceSn.getKey())){
                mesPart = mesPartMap.get(mesProduceSn.getKey());
            }else{
                //查询物料信息
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesProduceSn.getKey(), "partNo", ddlPackBean);
                mesPart = mesPartRDao.getByProperty(ddlPackBean);
                if(Objects.isNull(mesPart)){
                    throw new ImppBusiException(String.format("物料【%s】信息不存在", mesProduceSn.getKey()));
                }
                mesPartMap.put(mesProduceSn.getKey(),mesPart);
            }
            //根据物料获取已发布的工单
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getInPackArray(orderStatus, "workOrderStatus", ddlPackBean);
            MesWorkOrder oldMesWorkOrder = workOrderRepository.getByProperty(ddlPackBean);
            if(Objects.isNull(oldMesWorkOrder)){
                throw new ImppBusiException(String.format("物料【%s】状态未已发布的工单信息不存在", mesPart.getPartNo()));
            }
            //查询工作中心
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                throw new ImppBusiException(String.format("产线【%s】不存在", oldMesWorkOrder.getWorkCenterCode()));
            }
            //获取生产版本
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
            MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
            if (null == mesProductVersion) {
                throw new ImppBusiException(String.format("物料【%s】生产版本信息不存在", mesPart.getPartNo()));
            }
            //物料+生产版本获取bom信息
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesProductVersion.getAlternativePartList(), "bomVersion", ddlPackBean);
            List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesBoms)) {
                throw new ImppBusiException(String.format("物料【%s】生产版本【%s】对应bom信息不存在",
                        mesPart.getPartNo(), mesProductVersion.getProductVersion()));
            }
            oldMesWorkOrder.setNum(mesWorkOrderList.size());
            oldMesWorkOrder.setReportedQty(MathOperation.add(oldMesWorkOrder.getNum(), oldMesWorkOrder.getReportedQty()));
            //更新SAP计划完成数量
            saveMesProductPlan(oldMesWorkOrder, false, true);

            //更新工单状态
            double unCompleteQty = MathOperation.sub(oldMesWorkOrder.getQty(), oldMesWorkOrder.getReportedQty());
            oldMesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (oldMesWorkOrder.getReportedQty() > oldMesWorkOrder.getQty()) {
                throw new ImppBusiException(String.format("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                        oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty()));
            } else if (Objects.equals(oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty())) {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
            ConvertBean.serviceModelUpdate(oldMesWorkOrder,userName);
            workOrderRepository.update(oldMesWorkOrder);
            //保存数据
            List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
            MesProductOffLine newMesProductOffLine;
            for (MesProduceSn produceSn : mesWorkOrderList) {
                for (MesBom mesBom : mesBoms) {
                    newMesProductOffLine = new MesProductOffLine();
                    newMesProductOffLine.setReportPartNo(oldMesWorkOrder.getPartNo());
                    newMesProductOffLine.setReportPartNameRdd(oldMesWorkOrder.getPartName());
                    newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                    newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                    newMesProductOffLine.setItemQty(mesBom.getItemQty());
                    newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                    newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                    newMesProductOffLine.setQty(1d);
                    newMesProductOffLine.setReportSn(produceSn.getProductSn());
                    newMesProductOffLine.setBomVersion(oldMesWorkOrder.getProductVersion());
                    newMesProductOffLine.setSerialNumber(produceSn.getProductSn());
                    newMesProductOffLine.setUnit(mesBom.getUnit());
                    newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                    newMesProductOffLine.setWorkOrderNo(oldMesWorkOrder.getWorkOrderNo());
                    newMesProductOffLine.setWorkCenterCode(oldMesWorkOrder.getWorkCenterCode());
                    newMesProductOffLine.setWorkCellCode(oldMesWorkOrder.getWorkCellCode());
                    newMesProductOffLine.setReportType(oldMesWorkOrder.getReportType());
                    newMesProductOffLine.setSapWorkCenter(oldMesWorkOrder.getErpWorkCenter());
                    newMesProductOffLine.setOrganizeCode(oldMesWorkOrder.getOrganizeCode());

                    ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                    mesProductOffLineList.add(newMesProductOffLine);
                }
            }
            mesProductOffLineRDao.saveAll(mesProductOffLineList);
        }
    }
    public void doPcnJobProductReport(List<MesProduceSn> mesProduceSns,List<MesShift> mesShiftList,String organizeCode, String userName ,MesWorkOrder mesWorkOrderDb) {

        //查询物料信息
        DdlPackBean ddlPackBeanPart = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getPartNo(), "partNo", ddlPackBeanPart);
        MesPart mesPart = mesPartRDao.getByProperty(ddlPackBeanPart);
        if (null == mesPart) {
            MesPcnException.throwMesBusiException("物料【%s】信息不存在", mesWorkOrderDb.getPartNo());
        }

        //查询产线信息
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrderDb.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrderDb.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesWorkCenter)) {
            MesPcnException.throwMesBusiException("产线【%s】不存在", mesWorkOrderDb.getWorkCenterCode());
        }
        //获取生产版本
        MesProductVersion mesProductVersion = getMesProductVersion(mesWorkOrderDb);
        //物料+生产版本获取bom信息
        List<MesBom> mesBoms =  findBomList(mesPart.getOrganizeCode(), mesPart, mesProductVersion.getAlternativePartList());
        //报工类型
        if (MesExtEnumUtil.REPORT_TYPE.REPORT.getValue() == mesWorkOrderDb.getReportType()) {
            mesWorkOrderDb.setReportedQty(MathOperation.add(mesWorkOrderDb.getNum(), mesWorkOrderDb.getReportedQty()));
            //更新工单状态
            double unCompleteQty = MathOperation.sub(mesWorkOrderDb.getQty(), mesWorkOrderDb.getReportedQty());
            mesWorkOrderDb.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (mesWorkOrderDb.getReportedQty() > mesWorkOrderDb.getQty()) {
                MesPcnException.throwMesBusiException("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                        mesWorkOrderDb.getReportedQty(), mesWorkOrderDb.getQty());
            } else if (Objects.equals(mesWorkOrderDb.getReportedQty(), mesWorkOrderDb.getQty())) {
                mesWorkOrderDb.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                mesWorkOrderDb.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
            }
        } else {
            //报工调整数量加-
            mesWorkOrderDb.setAdjustQty((MathOperation.add(mesWorkOrderDb.getNum(), mesWorkOrderDb.getAdjustQty())));
            //冲销数量大于工单完成数量报错
            if (mesWorkOrderDb.getAdjustQty() > mesWorkOrderDb.getReportedQty()) {
                MesPcnException.throwMesBusiException("冲销数量【%s】大于工单完成数量【%s】,不允许冲销",
                        mesWorkOrderDb.getAdjustQty(), mesWorkOrderDb.getReportedQty());
            }
        }
        LOGGER.info("产线:{} 零件:{}更新工单", mesWorkOrderDb.getWorkCenterCode(), mesWorkOrderDb.getPartNo());
        //更新工单
        updateMesWorkOrder(mesWorkOrderDb, userName );

        //生成条码 todo 有条码 不生成

        //保存数据
//        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
//        for (MesBom mesBom : mesBoms) {
//            mesProductOffLineList.add(creatMesProductOffLine(mesWorkOrder, mesProductVersion, sn, mesBom, false));
//        }
//        mesProductOffLineService.insertBatch(mesProductOffLineList);

        LOGGER.info("产线:{} 零件:{}更新工单mesProduceSns:{}", mesWorkOrderDb.getWorkCenterCode(), mesWorkOrderDb.getPartNo(),mesProduceSns.size());
        //保存数据
        List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
        MesProductOffLine newMesProductOffLine;
        for (MesProduceSn produceSn : mesProduceSns) {
            for (MesBom mesBom : mesBoms) {
                newMesProductOffLine = new MesProductOffLine();
                newMesProductOffLine.setReportPartNo(mesWorkOrderDb.getPartNo());
                newMesProductOffLine.setReportPartNameRdd(mesWorkOrderDb.getPartName());
                newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                newMesProductOffLine.setItemPartName(mesBom.getItemPartName());
                newMesProductOffLine.setItemQty(mesBom.getItemQty());
                newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                newMesProductOffLine.setQty(1d);
                newMesProductOffLine.setReportSn(produceSn.getProductSn());
                newMesProductOffLine.setBomVersion(mesWorkOrderDb.getProductVersion());
                newMesProductOffLine.setSerialNumber(produceSn.getProductSn());
                newMesProductOffLine.setUnit(mesBom.getUnit());
                newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                newMesProductOffLine.setWorkOrderNo(mesWorkOrderDb.getWorkOrderNo());
                newMesProductOffLine.setWorkCenterCode(mesWorkOrderDb.getWorkCenterCode());
                newMesProductOffLine.setWorkCellCode(mesWorkOrderDb.getWorkCellCode());
                newMesProductOffLine.setReportType(mesWorkOrderDb.getReportType());
                newMesProductOffLine.setSapWorkCenter(mesWorkOrderDb.getErpWorkCenter());
                newMesProductOffLine.setOrganizeCode(organizeCode);

                ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                mesProductOffLineList.add(newMesProductOffLine);
            }
        }
        mesProductOffLineRDao.saveAll(mesProductOffLineList);
    }
    private void updateMesWorkOrder(MesWorkOrder mesWorkOrder, String userName ) {
        //修改工单，需要重新同步
        mesWorkOrder.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        ConvertBean.serviceModelUpdate(mesWorkOrder, userName);
        workOrderRepository.update(mesWorkOrder);
        //保存记录
        saveMesWorkOrderLog(mesWorkOrder);
    }
    public void saveMesWorkOrderLog(MesWorkOrder mesWorkOrder) {
        MesWorkOrderLog workOrderLog = new MesWorkOrderLog();
        BeanUtils.copyProperties(mesWorkOrder, workOrderLog, "id");
        mesWorkOrderLogRepository.insert(workOrderLog);
    }
    public List<MesBom> findBomList(String organizeCode, MesPart mesPart, String bomVersion) {
        List<MesBom> bomData = new ArrayList<>();
        // 递归展开 BOM
        expendBomChild(organizeCode, mesPart.getPartNo(), bomVersion, bomData);

        if (bomData.size() == 0) {
            MesPcnException.throwMesBusiException("零件号 %s 没找找到 ERP 同步的原始 BOM 数据，请检查接口是否正常！", mesPart.getPartNo());
        }

        Iterator<MesBom> iterator = bomData.iterator();
        while (iterator.hasNext()) {
            MesBom item = iterator.next();
            // 删除虚零件
            if (Objects.equals("X", item.getPartType())) {
                iterator.remove();
            } else {
                item.setPartNo(mesPart.getPartNo());
                item.setPartName(mesPart.getPartName());
            }
        }

        Map<String, MesBom> summaryBomMap = new HashMap<>();

        for (MesBom item : bomData) {
            String key = String.format("%s:%s", item.getPartNo().toUpperCase().trim(),
                    item.getItemPartNo().toUpperCase().trim());
            if (!summaryBomMap.containsKey(key)) {
                summaryBomMap.put(key, item);
            } else {
                MesBom tempItem = summaryBomMap.get(key);
                // 子零件重复的累加数量
                tempItem.setItemQty(MathOperation.add(tempItem.getItemQty(), item.getItemQty()));
            }
        }
        // 返回汇总数据
        return new ArrayList<>(summaryBomMap.values());
    }
    /**
     * 递归展开 BOM
     *
     * @param organizeCode 工厂代码
     * @param partNo       父零件号
     * @param bomData      存放 BOM 数据的集合
     */
    private void expendBomChild(String organizeCode, String partNo, String bomVersion, List<MesBom> bomData) {
        List<MesBom> bomDetails =  findMesBomByPartNoAndBomVersion(partNo, organizeCode, bomVersion);

        for (MesBom item : bomDetails) {
            MesBom newItem = new MesBom();
            BeanUtils.copyProperties(item, newItem);
            // 处理父零件用量不为 1 的情况
            if (newItem.getQty() != 1.0 && newItem.getQty() > 0.0) {
                Double rateQty = MathOperation.div(newItem.getItemQty(), newItem.getQty(), 8);
                newItem.setQty(1.0);
                newItem.setItemQty(rateQty);
            }
            // 加入到 BOM 清单中
            bomData.add(newItem);
            // 如果有子阶，继续递归
            if (Objects.equals("X", item.getPartType())) {
                expendBomChild(organizeCode, newItem.getItemPartNo(), null, bomData);
            }
        }
    }
    public List<MesBom> findMesBomByPartNoAndBomVersion(String partNo, String organizeCode, String bomVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(bomVersion, "bomVersion", ddlPackBean);
        DdlPreparedPack.getStringSmallerPack(TimeTool.getNowTime(true), "effStartTime", ddlPackBean);
        DdlPreparedPack.getOrderBy("effStartTime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), ddlPackBean);
        List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
        if (org.springframework.util.CollectionUtils.isEmpty(mesBoms)) {
            MesPcnException.throwMesBusiException("物料【%s】生产版本【%s】对应bom信息不存在", partNo, bomVersion);
        }
        MesBom next = mesBoms.iterator().next();
        //如果存在多个bomCode取最新的一个
        return mesBoms.stream().collect(Collectors.groupingBy(MesBom::getBomCode)).get(next.getBomCode());
    }
    private MesProductVersion getMesProductVersion(MesWorkOrder mesWorkOrder) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesWorkOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(mesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (null == mesProductVersion) {
            MesPcnException.throwMesBusiException("物料【%s】生产版本信息不存在", mesWorkOrder.getPartNo());
        }
        return mesProductVersion;
    }
    @Override
    public List<MesProduceSn> getUnReportMesProduceSn(String organizeCode) {
        List<MesProduceSn> mesProduceSnList = new ArrayList<>();
        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        List<Integer> statusList = Stream.of(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue(), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue()).collect(Collectors.toList());
        DdlPreparedPack.getInPackList( statusList,"reportStatus",ddlPackBean);
        DdlPreparedPack.getIsNotNull("workOrderNo",ddlPackBean);

        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"createDatetime"}, ddlPackBean);
        List<MesProductionRecord> mesProductionRecordList = productionRecordRao.findByHqlTopWhere(ddlPackBean, 5);
        if (mesProductionRecordList.isEmpty()){
            return mesProduceSnList;
        }
        //2.根据工单查询工单下的mesProduceSn
        List<String> workOrderNoList = mesProductionRecordList.stream().map(MesProductionRecord::getWorkOrderNo).collect(Collectors.toList());
        //工单号
        DdlPackBean snPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(workOrderNoList,"workOrderNo",snPackBean);
        return mesProduceSnRao.findByHqlWhere(snPackBean);
    }
    @Override
    public List<MesProductionRecord> getPreDayReportMesProductionRecord(String organizeCode,List<MesShift> mesShiftList, String  workCenterCode ) {

        //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode,"workCenterCode",ddlPackBean);
        DdlPreparedPack.getNumEqualPack( MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_50.getValue(),"reportStatus",ddlPackBean);
        //获取前一天的早班到当天早班的区间的所有加工记录，统计个数
        DdlPreparedPack.timeBuilder(TimeTool.getToday()+" 08:00:00",
                new SimpleDateFormat("yyyy-MM-dd").format(TimeTool.getDateBefore(new Date(),1))+" 08:00:00",
                "completeDateTime", ddlPackBean, false);

        return  productionRecordRao.findByHqlWhere(ddlPackBean);
    }
    @Override
    public List<MesProduceSn> getPreDayReportMesProduceSn(String organizeCode,List<MesProductionRecord> mesProductionRecordList ) {
        DdlPackBean snPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(mesProductionRecordList.stream().map(MesProductionRecord::getProductSn).collect(Collectors.toList()),
                "productSn",snPackBean);
        return mesProduceSnRao.findByHqlWhere(snPackBean);
    }

    private void saveMesProductPlan(MesWorkOrder bean, boolean isInsert, boolean isReport) {
        if (StringUtil.isEmpty(bean.getPlanOrderNo())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanStartTime(), "planStartDate", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanEndTime(), "planEndDate", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                MesProductPlan mesPlanOrder = new MesProductPlan();
                mesPlanOrder.setPlanOrderNo("");
                mesPlanOrder.setPlanQty(bean.getQty());
                mesPlanOrder.setPlanPartNo(bean.getPartNo());
                mesPlanOrder.setCompleteQty(0d);
                mesPlanOrder.setUncompleteQty(bean.getQty());
                mesPlanOrder.setUnit(bean.getUnit());
                mesPlanOrder.setPlanStartDate(bean.getPlanStartTime());
                mesPlanOrder.setPlanEndDate(bean.getPlanEndTime());
                mesPlanOrder.setPartMappingWorkCenterCode(bean.getErpWorkCenter());
                mesPlanOrder.setPlanOrganizeCode(bean.getOrganizeCode());
                mesPlanOrder.setOrganizeCode(bean.getOrganizeCode());
                ConvertBean.serviceModelInitialize(mesPlanOrder, bean.getCreateUser());
                mesProductPlanRDao.insert(mesPlanOrder);
            } else {
                if (isInsert) {
                    mesProductPlan.setPlanQty(MathOperation.add(mesProductPlan.getPlanQty(), bean.getQty()));
                } else {
                    //报工
                    if (isReport) {
                        mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                        //报工调整
                    } else {
                        mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                    }
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        } else {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPlanOrderNo(), "planOrderNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                throw new ImppBusiException(String.format("SAP计划单号【%s】物料【%s】,不存在", bean.getPlanOrderNo(), bean.getPartNo()));
            }
            if (!isInsert) {
                if (Objects.isNull(mesProductPlan.getCompleteQty())) {
                    mesProductPlan.setCompleteQty(0d);
                }
                //报工
                if (isReport) {
                    mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                    //报工调整
                } else {
                    mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        }
    }

    @Override
    public List<MesWorkOrder> insertMesWorkOrder(List<MesProduceSn> mesProduceSns,List<MesShift> mesShiftList, String organizeCode, String userName) {
        List<MesWorkOrder> mesWorkOrderList=new ArrayList<>();
        Map<String, List<MesProduceSn>> snListMap = mesProduceSns.stream().collect(Collectors.groupingBy(sn -> sn.getWorkCenterCode() +"="+ sn.getPartNo()));
        for (Map.Entry<String, List<MesProduceSn>> stringListEntry : snListMap.entrySet()) {
            List<MesProduceSn> value = stringListEntry.getValue();
            MesProduceSn mesProduceSn = value.get(0);
            String[] split = stringListEntry.getKey().split("=");
            String workCenterCode = split[0];
            String partNo = split[1];

            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                MesPcnException.throwMesBusiException("产线【%s】不存在",workCenterCode);
            }
            MesWorkOrder item=new MesWorkOrder();
            item.setOrganizeCode(organizeCode);
            item.setWorkCenterCode(workCenterCode);
            item.setPartNo(partNo);
            item.setWorkOrderType(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue());
            item.setOrderFlag(MesExtEnumUtil.ORDER_TYPE_IDENTIFICATION.N.getValue());
            List<MesShift> collect = mesShiftList.stream().filter(s -> s.getShiftCode().equalsIgnoreCase("1")).collect(Collectors.toList());
            MesShift mesShift= collect.get(0);
            item.setShiftCode(mesShift.getShiftCode());
            item.setShiftCode(mesShift.getShiftName());
            String today = TimeTool.getToday();
            item.setPlanStartTime(today+" "+mesShift.getStartTime()+":00");
            item.setPlanEndTime(today+" "+mesShift.getEndTime()+":00");

            //生成工单号
            List<String> orderList=new ArrayList<>();
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel("MES_WORK_CENTER_ORDER_NO");
            genSerialNoModel.setPartNo(workCenterCode);
            orderList = syncFuncService.syncSerialNo(genSerialNoModel,userName, organizeCode, 1).getResultList();

            String orderNo = orderList.get(0);
            DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(orderNo, "workOrderNo", orderPackBean);
            if (mesWorkCenterRDao.isExitByHql(orderPackBean)) {
                MesPcnException.throwMesBusiException("单号流水码生成重复");
            }
            item.setWorkOrderNo(orderNo);
            //校验标识
            //若工单类型为排序
            List<MesWorkOrderToWms> saveMesWorkOrderToWms = new ArrayList<>();
            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()) {
                //校验产线类型和工单类型是否匹配
                if (!Objects.equals(mesWorkCenter.getCenterType(), MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue())) {
                    MesPcnException.throwMesBusiException("产线【%s】类型为【%s】，请选择非排序产线", mesWorkCenter.getWorkCenterCode(),
                            MesExtEnumUtil.WORK_CENTER_TYPE.valueOfDescription(mesWorkCenter.getCenterType()));
                }

                //校验物料生产版本是否存在
                MesProductVersion mesProductVersion = checkMesProductVersion(item);
                item.setProductVersion(mesProductVersion.getProductVersion());
                //获取bom信息
                List<MesBom> bomList =  findMesBomByPartNoAndBomVersion(item.getPartNo(), item.getOrganizeCode(), mesProductVersion.getAlternativePartList());
                item.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue());
                //发送工单信息给WMS
                saveMesWorkOrderToWms.addAll(saveMesWorkOrderToWms(item, bomList));
            }
            //物料名称不存在，从物料信息中获取

            item.setPartName(mesProduceSn.getPartName());
            //生产时间新增取当天
            item.setProduceTime(TimeTool.getToday());
            item.setUnCompleteQty(0d);

//            if (item.getWorkOrderType() == MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue()){
//                //生产工单-非排序手工插单，关闭该零件当天之前日期非排序工单
//                List<String> partNoList = Collections.singletonList(item.getPartNo());
//                 deleteUnSortOrder(orderNo,partNoList);
//            }
            item.setQty(value.stream().mapToDouble(MesProduceSn::getQty).sum());
            item.setSystemSyncStatus(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            ConvertBean.serviceModelInitialize(item, userName);
            MesWorkOrder mesWorkOrder = workOrderRepository.insert(item);
            //保存发送WMS数据
            if (CollectionUtils.isNotEmpty(saveMesWorkOrderToWms)) {
                mesWorkOrderToWmsRDao.saveAll(saveMesWorkOrderToWms);
            }
            saveMesWorkOrderLog(mesWorkOrder);
            mesWorkOrderList.add(mesWorkOrder);
        }

        return mesWorkOrderList;
    }

    public void deleteUnSortOrder(String organizeCode, List<String> partNoList) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(partNoList, "partNo", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.ORDER_TYPE.UN_SORT.getValue(), "workOrderType", ddlPackBean);
        DdlPreparedPack.getStringSmallerNotEqualPack(TimeTool.getToday(), "planStartTime", ddlPackBean);
        DdlPreparedPack.getNumberSmallerPack(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue(), "workOrderStatus", ddlPackBean);
        workOrderRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime", "workOrderStatus"},
                new Object[]{"CLOSE_JOB", TimeTool.getNowTime(true), MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue()}, ddlPackBean);
    }
    public List<MesWorkOrderToWms> saveMesWorkOrderToWms(MesWorkOrder mesWorkOrder, List<MesBom> mesBomList) {
        List<MesWorkOrderToWms> mesWorkOrderToWmsList = new ArrayList<>();
        MesWorkOrderToWms mesWorkOrderToWms = null;
        for (MesBom mesBom : mesBomList) {
            mesWorkOrderToWms = new MesWorkOrderToWms();
            BeanUtils.copyProperties(mesWorkOrder, mesWorkOrderToWms, "id");
            mesWorkOrderToWms.setUnit(mesBom.getUnit());
            mesWorkOrderToWms.setItemPartNo(mesBom.getItemPartNo());
            mesWorkOrderToWms.setItemPartName(mesBom.getItemPartName());
            mesWorkOrderToWms.setItemUnit(mesBom.getItemUnit());
            mesWorkOrderToWms.setItemQty(MathOperation.mul(mesWorkOrder.getQty(), mesBom.getItemQty()));
            ConvertBean.serviceModelInitialize(mesWorkOrderToWms, mesWorkOrderToWms.getCreateUser());
            mesWorkOrderToWmsList.add(mesWorkOrderToWms);
        }
        return mesWorkOrderToWmsList;
    }
    private MesProductVersion checkMesProductVersion(MesWorkOrder item) {
        DdlPackBean ddlPackBean;
        ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartNo(), "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getErpWorkCenter(), "workCenterCode", ddlPackBean);
        MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProductVersion)) {
            MesPcnException.throwMesBusiException("物料【%s】产线【%s】生产版本【%s】信息不存在", item.getPartNo(), item.getWorkCenterCode(), item.getProductVersion());
        }
        return mesProductVersion;
    }
}
